/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is openCRX.
 *
 * The Initial Developer of the Original Code is
 * Josh Geenen <openCRX@pirules.net>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
window.addEventListener("load", function optionsLoadListener(e) {
  oInitialize();
 }, false);
var usernames = {};
/**
 * initialize
 * Initializes the string bundle, oFileIO and oPreferences scripts and fills the
 * login tree.
 */
function oInitialize() {
  oFileIO.init();
  oPreferences.getSyncPrefs();
  if (document.getElementById("loginTree"))
    oFillLoginTree();
}

/**
 * oFillLoginTree
 * Populates the login tree for the preferences window with the synchronized
 * accounts and directories.
 */
function oFillLoginTree() {
  var tree = document.getElementById("loginTree");
  var treechildren = document.getElementById("loginTreeChildren");
  if (treechildren)
    try { tree.removeChild(treechildren); } catch(e) {}
  var newTreeChildren = document.createElement("treechildren");
  newTreeChildren.setAttribute("id", "loginTreeChildren");
  tree.appendChild(newTreeChildren);
  var abs = oAbManager.getSyncedAddressBooks();
  for (var i in abs) { // i is the URL
    var abName = "";
    if (abs[i])
      abName = abs[i].getName();
    usernames[i] = true;
    oAddLoginToTree(newTreeChildren, i, abName)
  }
}
/**
 * oRemoveSelectedLogin
 * Removes the selected account's username and auth token from the login manager.
 */
function oRemoveSelectedLogin() {
  var tree = document.getElementById("loginTree");
  if (!tree || tree.currentIndex == -1) // stop if nothing is selected
    return;
  var cellText = tree.view.getCellText(tree.currentIndex, tree.columns.getColumnAt(0));
  if (cellText && confirm(oStringBundle.getStr("removeLogin"))) {
    usernames[cellText] = null;
    // remove the saved prefs from the address books
    var abs = oAbManager.getSyncedAddressBooks();
    var abObj = abs[cellText];
    if (abObj) {
      abObj.setUrl("");
      // XXX set last dates to 0
    }
    var treeitem = document.getElementById(cellText);
    var treechildren = document.getElementById("loginTreeChildren");
    if (treeitem && treechildren)
      try { treechildren.removeChild(treeitem); } catch(e) {}
  }
}

/**
 * oAddLogin
 * Adds an auth token to the login manager.
 */
function oAddLogin() {
  var prompt = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                .getService(Ci.nsIPromptService)
                .prompt;
  var url = {};

  // opens a username/password prompt
  var ok = prompt(window, oStringBundle.getStr("addUrlTitle"),
                  oStringBundle.getStr("addUrl"), url, null,
                  {value: false});
  if (!ok)
    return;
  
  if (usernames[url.value]) { // the username already exists
    alert(oStringBundle.getStr("usernameExists")); // XXX change string
    return;
  }
  
  // XXX check validity of URL
  oAddAb(url.value);
}

function oAddAb(aUrl) {
  
  var input = {};
  var prompt = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                .getService(Ci.nsIPromptService)
                .prompt;
  var ok = prompt(window, oStringBundle.getStr("abNameTitle"),
                  oStringBundle.getStr("abName"), input, null,
                  {value: false});

  if (ok && input.value && input.value != "") {
    if (oAbManager.getAbByName(input.value, true)) {
      // if an address book already exists with the name, warn the user and let
      // him or her choose a new name
      if(!confirm(StringBundle.getStr("abExists"))) {
        oAddAb(aUrl);
        return;
      }
    }
    usernames[aUrl] = true;
    var ab = new oAddressBook(oAbManager.getAbByName(input.value));
    ab.setUrl(aUrl);
    // XXX set last upload/download date to 0
    var treechildren = document.getElementById("loginTreeChildren");
    oAddLoginToTree(treechildren, aUrl, input.value);
  }
  else if (!ok)
    oLOGGER.VERBOSE_LOG("prompt canceled");
  else
    oLOGGER.VERBOSE_LOG("Invalid input: " + input ? input.value : null);
}

/**
 * oAddLoginToTree
 * Adds login information (username and directory name) to the tree.
 * @param aTreeChildren The <treechildren> XUL element.
 * @param aUsername     The username (e-mail address).
 * @param aDirName      The name of the directory with which this account's
 *                      contacts will be synchronized.
 */
function oAddLoginToTree(aTreeChildren, aUsername, aDirName) {
  var treeitem = document.createElement("treeitem");
  var treerow = document.createElement("treerow");
  treeitem.setAttribute("id", aUsername);
  var username = document.createElement("treecell");
  username.setAttribute("label", aUsername);
  var addressbook = document.createElement("treecell");

  addressbook.setAttribute("label", aDirName);
  treerow.appendChild(username);
  treerow.appendChild(addressbook);
  treeitem.appendChild(treerow);
  aTreeChildren.appendChild(treeitem);
}
/**
 * oChangeAbName
 * Changes the name of the selected address book, if an address book with the
 * new name does exist, otherwise it updates the preferences to synchronize the
 * other address book instead.
 */
function oChangeAbName() {
  var tree = document.getElementById("loginTree");
  if (!tree || tree.currentIndex == -1) // stop if nothing is selected
    return;
  var username = tree.view.getCellText(tree.currentIndex, tree.columns.getColumnAt(0));
  var oldAb = oAbManager.getSyncedAddressBooks()[username];
  if (oldAb)
    oldAb = oldAb.primary;
  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                        .getService(Components.interfaces.nsIPromptService);
  // set the default to the existing name or, if not present, the username
  var input = {value: oldAb && oldAb.mDirectory ? oldAb.mDirectory.dirName : username};
  var check = {};
  var result = prompts.prompt(null, oStringBundle.getStr("abNameTitle") + " " +
                              username, oStringBundle.getStr("abName"), input,
                              null, check);
  // change the synced address book if:
  //  * the user clicked OK
  //  * there was a valid, non-blank response
  //  * the new name isn't the same as the old name
  if (result && input.value && input.value != "" &&
      !(oldAb && input.value == oldAb.mDirectory.dirName)) {
    var existingAb = oAbManager.getAbByName(input.value, true);
    var newAb;
    if (existingAb) {
      // if an ab with the name already exists, warn the user and let him or her
      // choose a new name
      if(!confirm(oStringBundle.getStr("abExists"))) {
        oChangeAbName();
        return;
      }
      // if they still want it, setup the new one and remove the prefs from the old
      existingAb.dirName = input.value;
      newAb = new oAddressBook(existingAb);
      // remove the prefs from the old address book
      if (oldAb) {
        oldAb.setUsername("");
        oldAb.setPrimary(false);
        oldAb.setLastSyncDate(0);
      }
      // setup the prefs for the new address book
      newAb.setUsername(username);
      newAb.setPrimary(true);
      newAb.setLastSyncDate(0);
    }
    // rename the old address book, if present, and don't change any prefs
    else if (oldAb) {
      // if the old address book is either the PAB or CAB, don't rename and
      // make a new address book instead
      if (oldAb.mURI == "moz-abmdbdirectory://abook.mab" ||
          oldAb.mURI == "moz-abmdbdirectory://history.mab") {
        newAb = new oAddressBook(oAbManager.getAbByName(input.value));
        // setup the prefs for the new address book
        newAb.setUsername(username);
        newAb.setPrimary(true);
        newAb.setLastSyncDate(0);
        // remove the prefs from the old one
        oldAb.setUsername("");
        oldAb.setPrimary(false);
        oldAb.setLastSyncDate(0);
      }
      else {
        try {
          // this will only fail if input.value is the PAB or CAB's name, which
          // should not ever happen since the PAB and CAB should already exist
          oldAb.setName(input.value);
        }
        catch(e) {
          oLOGGER.LOG_WARNING("Attempt to rename a directory to the PAB or CAB aborted");
          alert(oStringBundle.getStr("invalidDirName"));
          oChangeAbName();
        }
      }
    }
    // otherwise, make the new address book
    else {
      newAb = new oAddressBook(oAbManager.getAbByName(input.value));
      // setup the prefs for the new address book
      newAb.setUsername(username);
      newAb.setPrimary(true);
      newAb.setLastSyncDate(0);
    }
    tree.view.setCellText(tree.currentIndex, tree.columns.getColumnAt(1), input.value);
  }
}
